/**
 * @license
 * Copyright 2020 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */

const localRunConfig = {
  reporters: ['progress'],
  plugins: ['karma-jasmine', 'karma-chrome-launcher'],
  browsers: ['Chrome']
};

function getBrowserStackConfig() {
  const browserstackConfig = {
    hostname: 'bs-local.com',
    plugins: ['karma-jasmine', 'karma-browserstack-launcher'],
    reporters: ['progress', 'BrowserStack'],
    port: 9812,
    browserStack: {
      username: process.env.BROWSERSTACK_USERNAME,
      accessKey: process.env.BROWSERSTACK_ACCESS_KEY,
      apiClientEndpoint: 'https://api.browserstack.com',
      tunnelIdentifier: `e2e_browserstack_benchmark_${Date.now()}_${
          Math.floor(Math.random() * 1000)}`
    },
    customLaunchers: {},
    browsers: []
  };

  // The JSON file `./browsers.json` stores the `customLaunchers`. The key is
  // tabId, while the value is specific browser setting.
  const browsers = require('./browsers.json');
  browserstackConfig.customLaunchers = browsers;
  return browserstackConfig;
}

module.exports = function(config) {
  let extraConfig = null;
  if (config.browserstack) {
    extraConfig = getBrowserStackConfig();
  } else {
    extraConfig = localRunConfig;
  }

  let localBuildableFiles = [
    'tfjs-core/dist/tf-core.js',
    'tfjs-backend-cpu/dist/tf-backend-cpu.js',
    'tfjs-backend-webgl/dist/tf-backend-webgl.js',
    'tfjs-layers/dist/tf-layers.js',
    'tfjs-converter/dist/tf-converter.js',
    'tfjs-backend-wasm/dist/tf-backend-wasm.js',
    'tfjs-automl/dist/tf-automl.js',
  ].map(url => {
    let name = url.split('/')[0].replace('tfjs-', '').replace('backend-', '');
    if (config.localBuild?.includes(name)) {
      return `../../../dist/bin/${url}`;
    } else {
      return `https://unpkg.com/@tensorflow/${url.replace('/', '@latest/')}`;
    }
  });

  let files = localBuildableFiles.concat([
    'https://cdn.jsdelivr.net/npm/@tensorflow-models/universal-sentence-encoder@latest/dist/universal-sentence-encoder.min.js',
    'https://cdn.jsdelivr.net/npm/@tensorflow-models/speech-commands@latest/dist/speech-commands.min.js',
    'https://cdn.jsdelivr.net/npm/@tensorflow-models/posenet@latest/dist/posenet.min.js',
    'https://cdn.jsdelivr.net/npm/@tensorflow-models/body-pix@latest/dist/body-pix.min.js',
    'https://cdn.jsdelivr.net/npm/@tensorflow-models/pose-detection@latest/dist/pose-detection.min.js',
    {pattern: './benchmark_parameters.json', included: false, served: true},
    '../util.js', '../benchmark_util.js', '../model_config.js',
    'benchmark_models.js'
  ]);

  config.set({
    ...extraConfig,
    frameworks: ['jasmine'],
    files: files,
    preprocessors: {},
    singleRun: true,
    captureTimeout: 3e5,
    reportSlowerThan: 500,
    browserNoActivityTimeout: 1.2e5,
    browserDisconnectTimeout: 1.2e5,
    browserDisconnectTolerance: 0,
    browserSocketTimeout: 1.2e5,
    client: {jasmine: {random: false}},

    // The following configurations are generated by karma
    colors: true,
    logLevel: config.LOG_INFO,
    autoWatch: false,
    concurrency: Infinity
  })
}
